<?php

declare(strict_types=1);

namespace Drupal\migrate_qa\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;

/**
 * Migrate QA Connector Generator.
 *
 * @ConfigEntityType(
 *   id = "mqa_connector_generator",
 *   label = @Translation("Connector Generator"),
 *   handlers = {
 *     "list_builder" = "Drupal\migrate_qa\Controller\ConnectorGeneratorListBuilder",
 *     "form" = {
 *       "default" = "Drupal\migrate_qa\Form\ConnectorGeneratorForm",
 *       "add" = "Drupal\migrate_qa\Form\ConnectorGeneratorForm",
 *       "edit" = "Drupal\migrate_qa\Form\ConnectorGeneratorForm",
 *       "delete" = "Drupal\migrate_qa\Form\ConnectorGeneratorDeleteForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *     },
 *   },
 *   config_prefix = "connector_generator",
 *   admin_permission = "administer migrate_qa_connector generator",
 *   entity_keys = {
 *     "id" = "id",
 *     "tracker_migration" = "tracker_migration",
 *     "process_extra" = "process_extra"
 *   },
 *   config_export = {
 *     "id",
 *     "tracker_migration",
 *     "process_extra",
 *   },
 *   links = {
 *     "add-form" = "/admin/structure/migrate-qa/connector/settings/generators/add",
 *     "edit-form" = "/admin/structure/migrate-qa/connector/settings/generators/{mqa_connector_generator}/edit",
 *     "delete-form" = "/admin/structure/migrate-qa/connector/settings/generators/{mqa_connector_generator}/delete",
 *     "collection" = "/admin/structure/migrate-qa/connector/settings/generators",
 *   }
 * )
 */
class ConnectorGenerator extends ConfigEntityBase implements ConnectorGeneratorInterface {

  /**
   * Tracker migration that this config relates to.
   *
   * @var string
   */
  public $tracker_migration;

  /**
   * Extra Yaml to include in the migration process section.
   *
   * @var string
   */
  public $process_extra;

  /**
   * Utils for Generators
   *
   * @var \Drupal\migrate_qa\GeneratorUtil
   */
  public $generator_util;

  /**
   * Construct a ConnectorGenerator object.
   *
   *  {@inheritdoc}
   */
  public function __construct(array $values, $entity_type) {
    parent::__construct($values, $entity_type);
    // TODO Use dependency injection.
    $this->generator_util = \Drupal::service('migrate_qa.generator_util');
  }

  /**
   * Save the generator.
   *
   * @return int|null
   *   The response of parent::save().
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function save() {
    if ($this->isNew()) {
      $this->set('id', $this->tracker_migration);
    }
    // Set dependency on the tracker migration.
    $tracker_migration = $this->get('tracker_migration');
    $dependencies['enforced']['config'][] = 'migrate_plus.migration.' . $tracker_migration;
    $this->set('dependencies', $dependencies);
    $status = parent::save();

    // Save connector migration config.
    $yaml_string = $this->generator_util->getConfigTemplate('connector.yml');
    $yaml_string = str_replace('$migration', $this->tracker_migration, $yaml_string);
    $yaml_string = str_replace('$tracker_migration', $this->tracker_migration, $yaml_string);
    // Derive target type and id key from the connector generator.
    // Currently using its name.
    // TODO Use something more reliable, since entity type names can have `_`.
    $name_parts = explode('_', $this->tracker_migration);
    $entity_type_manager = $this->entityTypeManager();
    $target_type = NULL;
    // Tracker migrations start with one of the following:
    // `tc_` - for tracking based on manually created content
    // `tm_` for tracking based on content that is generated by a migration.
    if (str_ends_with($name_parts[0], 'c')) {
      $target_type = $name_parts[1];
    }
    if (str_ends_with($name_parts[0], 'm')) {
      // Get target entity type from the tracker generator config.
      // Tracker generator config: migrate_qa.tracker.beer_term.
      $tracker_generator_migration_storage = $entity_type_manager->getStorage('mqa_tracker_generator_migration');
      // Remove the first element of the array. The remainder is the name of the
      // associated tracker generator config.
      array_shift($name_parts);
      $tracker_generator_name = implode('_', $name_parts);
      $tracker_generator = $tracker_generator_migration_storage->load($tracker_generator_name);
      $target_type = $tracker_generator->get('entity_type');
      $override_source_id_key  = $tracker_generator->get('source_id_key');
      // Save the migration name, to set a migration lookup using the associated
      // migration.
      $content_migration_name = $tracker_generator_name;
    }
    if (!$target_type) {
      // TODO log an error.
      return $status;
    }
    // Get id key from the target type.
    $id_key = $entity_type_manager->getDefinition($target_type)->get('entity_keys')['id'];
    $yaml_string = str_replace('$target_type', $target_type, $yaml_string);
    $yaml_string = str_replace('$id_key', $id_key, $yaml_string);
    $yaml_string = str_replace('$connector_generator', $this->id(), $yaml_string);
    $source_id_key = $override_source_id_key ?? $id_key;
    $yaml_string = str_replace('$source_id_key', $source_id_key, $yaml_string);
    $migration_config_array = $this->generator_util->yamlStringToArray($yaml_string);
    // Use a migration lookup for the content id.
    if (isset($content_migration_name)) {
      $migration_config_array['process']['content/target_id'] = [
        'plugin' => 'migration_lookup',
        'migration' => $content_migration_name,
        'source' => $source_id_key,
      ];
    }
    if (is_string($this->process_extra)) {
      $migration_config_array = $this->generator_util->addProcessExtra($migration_config_array, $this->process_extra);
    }

    // Add the source migration config.
    $migration_config_array = $this->generator_util->addSource($migration_config_array, $this->tracker_migration);
    $this->generator_util->saveMigrationConfig($migration_config_array);

    return $status;
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    // Fall back to the entity ID if the default label is empty.
    $label = parent::label();
    if (!$label) {
      $label = $this->id();
    }
    return $label;
  }

}
